@@ -51,7 +51,7 @@ from debian.deb822 import Deb822
 from debmutate.reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 82)
+__version__ = (0, 84)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'confident', 'likely', 'possible', None]
 DEFAULT_MINIMUM_CERTAINTY = 'certain'
@@ -1064,18 +1064,3 @@ def min_certainty(certainties: Sequence[str]) -> str:
     return confidence_to_certainty(
         max([certainty_to_confidence(c)
             for c in certainties] + [0]))
-
-
-def load_renamed_tags():
-    import json
-    path = os.path.abspath(os.path.join(
-        os.path.dirname(__file__), '..', 'renamed-tags.json'))
-    if not os.path.isfile(path):
-        import pkg_resources
-        path = pkg_resources.resource_filename(
-            __name__, 'lintian-brush/renamed-tags.json')
-        if not os.path.isfile(path):
-            # Urgh.
-            path = '/usr/share/lintian-brush/renamed-tags.json'
-    with open(path, 'rb') as f:
-        return json.load(f)
