@@ -17,28 +17,28 @@
 
 """Utility functions for dealing with lintian overrides files."""
 
-import collections
+from debian.changelog import Version
 import os
+from typing import Optional, Iterator, Callable, List, Tuple
+
+from debmutate.lintian_overrides import (
+    LintianOverridesEditor,
+    LintianOverride,
+    iter_overrides,
+    )
 
 
-# https://lintian.debian.org/manual/section-2.4.html
-# File format (as documented in policy 2.4.1):
-# [[<package>][ <archlist>][ <type>]: ]<lintian-tag>[ [*]<lintian-info>[*]]
-
-
-VALID_TYPES = ['udeb', 'source', 'binary']
-Override = collections.namedtuple(
-    'Override', ['package', 'archlist', 'type', 'tag', 'info'])
-Override.__new__.__defaults__ = (None,) * len(Override._fields)  # type: ignore
-
-
-def overrides_paths():
+def overrides_paths() -> Iterator[str]:
     for path in ['debian/source/lintian-overrides']:
         if os.path.exists(path):
             yield path
+    if os.path.exists('debian'):
+        for entry in os.scandir('debian'):
+            if entry.name.endswith('.lintian-overrides'):
+                yield entry.path
 
 
-def update_overrides(cb):
+def update_overrides(cb: Callable[[LintianOverride], LintianOverride]) -> None:
     """"Call update_overrides_file on all overrides files.
 
     Args:
@@ -48,72 +48,9 @@ def update_overrides(cb):
         update_overrides_file(cb, path=path)
 
 
-def parse_override(line):
-    """Parse an override line
-
-    Args:
-      line: Line to parse
-    Returns:
-      An Override object
-    Raises:
-      ValueError: when encountering invalid syntax
-    """
-    line = line.strip()
-    archlist = None
-    package = None
-    type = None
-    if ': ' in line:
-        origin, issue = line.split(': ', 1)
-        while origin:
-            origin = origin.strip()
-            if origin.startswith('['):
-                archlist, origin = origin[1:].split(']', 1)
-            else:
-                try:
-                    field, origin = origin.split(' ', 1)
-                except ValueError:
-                    field = origin
-                    origin = ''
-                if field in VALID_TYPES:
-                    type = field
-                else:
-                    package = field
-    else:
-        issue = line
-    try:
-        tag, info = issue.split(None, 1)
-    except ValueError:
-        tag = issue
-        info = None
-    return Override(
-        package=package, archlist=archlist, type=type, tag=tag, info=info)
-
-
-def serialize_override(override):
-    """Serialize an override.
-
-    Args:
-      override: An Override object
-    Returns:
-      serialized override, including newline
-    """
-    origin = []
-    if override.package:
-        origin.append(override.package)
-    if override.archlist:
-        origin.append('[' + override.archlist + ']')
-    if override.type:
-        origin.append(override.type)
-    if origin:
-        line = ' '.join(origin) + ': ' + override.tag
-    else:
-        line = override.tag
-    if override.info:
-        line += ' ' + override.info
-    return line + '\n'
-
-
-def update_overrides_file(cb, path='debian/source/lintian-overrides'):
+def update_overrides_file(
+        cb: Callable[[LintianOverride], LintianOverride],
+        path: str = 'debian/source/lintian-overrides') -> bool:
     """Modify the overrides in a file.
 
     Args:
@@ -122,75 +59,74 @@ def update_overrides_file(cb, path='debian/source/lintian-overrides'):
     Returns:
         Whether the file was modified
     """
-    changed = False
-    lines = []
-    with open(path, 'r') as f:
-        for line in f.readlines():
-            if line.startswith('#') or not line.strip():
-                lines.append(line)
+    with LintianOverridesEditor(path=path) as editor:
+        new_lines = []
+        comments: List[str] = []
+        for entry in editor.lines:
+            if isinstance(entry, LintianOverride):
+                entry = cb(entry)
+                if entry is not None:
+                    new_lines.extend(comments)
+                    new_lines.append(entry)
+                comments = []
             else:
-                old = parse_override(line)
-                new = cb(old)
-                if old != new:
-                    changed = True
-                    if new is not None:
-                        lines.append(serialize_override(new))
-                else:
-                    lines.append(line)
-
-    if not changed:
-        return False
-
-    if lines:
-        with open(path, 'w') as f:
-            f.writelines(lines)
-    else:
-        os.unlink(path)
-
-    return True
+                comments.append(entry)
+        editor._parsed = new_lines + comments
+        return editor.has_changed()
 
 
-def _get_overrides(package=None):
-    if package in ('source', None):
-        paths = ['debian/source/lintian-overrides',
-                 'debian/source.lintian-overrides']
-    else:
-        paths = ['debian/%s.lintian-overrides' % package]
+def get_overrides(
+        type: Optional[str] = None,
+        package: Optional[str] = None) -> Iterator[LintianOverride]:
+    paths = []
+    if type in ('source', None):
+        paths.extend(
+            ['debian/source/lintian-overrides',
+             'debian/source.lintian-overrides'])
+    if type in ('binary', None):
+        if package is not None:
+            paths.extend(['debian/%s.lintian-overrides' % package])
+        elif os.path.isdir('debian'):
+            paths.extend(
+                [e.path for e in os.scandir('debian')
+                 if e.name.endswith('.lintian-overrides')])
 
     for path in paths:
         try:
             with open(path, 'r') as f:
-                for line in f.readlines():
-                    if line.startswith('#') or not line.strip():
-                        pass
-                    else:
-                        yield parse_override(line)
+                yield from iter_overrides(f)
         except FileNotFoundError:
             pass
 
 
-def override_exists(tag, info=None, package=None):
+def override_exists(
+        tag: str, info: Optional[str] = None,
+        package: Optional[str] = None,
+        type: Optional[str] = None,
+        arch: Optional[str] = None) -> bool:
     """Check if a particular override exists.
 
     Args:
       tag: Tag name
       info: Optional info
-      package: Package (as type, name tuple)
+      package: Package
+      type: package type (source, binary)
+      arch: Architecture
     """
-    for override in _get_overrides(package):
-        if override.tag != tag:
-            continue
-        if override.info and info != override.info:
-            continue
-        return True
+    for override in get_overrides(type=type, package=package):
+        if override.matches(package=package, info=info, tag=tag, arch=arch,
+                            type=type):
+            return True
     return False
 
 
-async def get_unused_overrides(packages):
+async def get_unused_overrides(
+        packages: List[Tuple[str, str]]
+        ) -> List[Tuple[str, str, Version, str]]:
     from .udd import connect_udd_mirror
     udd = await connect_udd_mirror()
 
-    args = []
+    args: List[str] = []
     extra = []
     for (type, name) in packages:
         extra.append('package = $%d AND package_type = $%d' % (
@@ -204,7 +140,10 @@ from lintian where tag = 'unused-override' AND (%s)""" % " OR ".join(extra),
         *args))
 
 
-def remove_unused():
+unused_overrides = None
+
+
+def remove_unused() -> List[LintianOverride]:
     from debian.deb822 import Deb822
     packages = []
     with open('debian/control', 'r') as f:
@@ -236,7 +175,9 @@ def remove_unused():
         global unused_overrides
         if unused_overrides is None:
             import asyncio
-            unused_overrides = asyncio.run(get_unused_overrides(packages))
+            loop = asyncio.get_event_loop()
+            unused_overrides = loop.run_until_complete(
+                get_unused_overrides(packages))
         if is_unused(override, unused_overrides):
             removed.append(override)
             return None
@@ -245,6 +186,21 @@ def remove_unused():
     return removed
 
 
+def load_renamed_tags():
+    import json
+    path = os.path.abspath(os.path.join(
+        os.path.dirname(__file__), '..', 'renamed-tags.json'))
+    if not os.path.isfile(path):
+        import pkg_resources
+        path = pkg_resources.resource_filename(
+            __name__, 'lintian-brush/renamed-tags.json')
+        if not os.path.isfile(path):
+            # Urgh.
+            path = '/usr/share/lintian-brush/renamed-tags.json'
+    with open(path, 'rb') as f:
+        return json.load(f)
+
+
 if __name__ == '__main__':
     import argparse
     parser = argparse.ArgumentParser()
